@@ -34,27 +34,25 @@
 #define __MATLABINT_H
 
 #include <matlabint_std.h>
+#include <set>
 #include <getfem_mesh.h>
 #include <getfem_mesh_fem.h>
-#include <bgeot_smatrix.h>
-
+#include <gmm.h>
+#include <mex.h>
 
 namespace matlabint
 {
-  typedef bgeot::smatrix<getfem::scalar_type> gf_sparse_matrix;
+  typedef gmm::row_matrix<gmm::wsvector<getfem::scalar_type> >
+  gf_sparse_by_row;
+  typedef gmm::col_matrix<gmm::wsvector<getfem::scalar_type> >
+  gf_sparse_by_col;
+  typedef gmm::csc_matrix_ref<const double *, const int *, const int *> 
+  gf_sparse_matlab_const_ref;
+
+  typedef bgeot::dim_type dim_type;
 
   class matlabint_mesh;
-
-  /* fake class for storing the dimension of unkowns with the mesh_fem
-     object */
-  class mesh_fem_int : public getfem::mesh_fem {
-    int u_dim;
-  public:
-    mesh_fem_int(getfem::getfem_mesh &m, unsigned _u_dim) : getfem::mesh_fem(m), u_dim(_u_dim) {}
-    unsigned get_u_dim() const { return u_dim; }
-  };
-
-
+  class matlabint_mesh_fem;
 
   /* fake vector class for matlab arrays (necessary for template functions) */
   class mlab_vect {
@@ -64,6 +62,9 @@ namespace matlabint
 		       p is a pseudo third dimension
 		    */
   public:
+    typedef double* iterator;
+    typedef const double* const_iterator;
+
     double& operator[](unsigned i) { if (i >= sz) THROW_INTERNAL_ERROR; return data[i]; }
     const double& operator[](unsigned i) const { if (i >= sz) THROW_INTERNAL_ERROR; return data[i]; }
     double& operator()(unsigned i, unsigned j, unsigned k=0) { if (i+j*m+k*m*n >= sz) THROW_INTERNAL_ERROR; 
@@ -71,18 +72,22 @@ namespace matlabint
     const double& operator()(unsigned i, unsigned j, unsigned k=0) const { if (i+j*m+k*m*n >= sz) THROW_INTERNAL_ERROR;return data[i+j*m+k*m*n]; }
 
     mlab_vect(): sz(0),data(0),m(0),n(0),p(0) {}
-    mlab_vect(const mxArray *mx) { 
+    void assign(const mxArray *mx) { 
       if (!mxIsDouble(mx)) THROW_INTERNAL_ERROR;
       data = mxGetPr(mx); sz = mxGetNumberOfElements(mx); 
       int ndim = mxGetNumberOfDimensions(mx);
       if (ndim < 3) {
 	m = mxGetM(mx); n = mxGetN(mx); p = 1;
-      } else if (ndim == 3) {
+      } else if (ndim >= 3) {
 	const int *dim = mxGetDimensions(mx);
-	m = dim[0]; n = dim[1]; p = dim[2];
+	m = dim[0]; n = dim[1]; p = 1;
+	for (int i=2; i < ndim; ++i) p*=dim[i];
       } else THROW_INTERNAL_ERROR;
       if (n*m*p != sz) THROW_INTERNAL_ERROR;
     }
+    mlab_vect(const mxArray *mx) { 
+      assign(mx);
+    }
     mlab_vect(double *v, int _sz) { data = v; sz = _sz; m=_sz; n=1; p = sz ? sz/(m*n) : 0; }
     unsigned size() const { return sz; }
     unsigned getm() const { return m; }
@@ -106,31 +111,74 @@ namespace matlabint
       return M;
     }
     bool in_range(double vmin, double vmax);
-  };
 
+    iterator begin() { return data; }
+    iterator end() { return data+sz; }
+    const_iterator begin() const { return data; }
+    const_iterator end() const { return data+sz; }
+
+    /* copies the matlab vector into a new VECT object */
+    template<class VECT> VECT to_vector() const {
+      VECT v(sz);
+      std::copy(begin(), end(), v.begin());
+      return v;
+    }
+  };
+}
+
+/* intermde: interface gmm pour les mlab_vect */
+namespace gmm {
+  template<> struct linalg_traits<matlabint::mlab_vect> {
+    typedef matlabint::mlab_vect this_type;
+    typedef linalg_false is_reference;
+    typedef abstract_vector linalg_type;
+    typedef double value_type;
+    typedef double& reference;
+    typedef this_type::iterator iterator;
+    typedef this_type::const_iterator const_iterator;
+    typedef abstract_plain storage_type;
+    typedef plain_access<iterator,const_iterator> access_type;
+    typedef plain_clear<iterator> clear_type;
+    static size_type size(const this_type &v) { return v.size(); }
+    static iterator begin(this_type &v) { return v.begin(); }
+    static const_iterator begin(const this_type &v) { return v.begin(); }
+    static iterator end(this_type &v) { return v.end(); }
+    static const_iterator end(const this_type &v) { return v.end(); }
+    static const void* origin(const this_type &v) { return &v; }
+    static void do_clear(this_type &v) { clear_type()(origin(v), begin(v), end(v)); }
+  };
+} /* fin de l'intermde */
+
+
+namespace matlabint {
+
+  /* input argument of the gf_* mex-files */
   class mexarg_in {
     void check_int_values(int vmin=INT_MIN, int vmax=INT_MAX);
+    double _to_scalar(bool isint);
   public:
     const mxArray *arg;
     int argnum;
 
     mexarg_in(const mxArray *_arg, int _num) { arg = _arg; argnum = _num; }
     bool                        is_string() { return (mxIsChar(arg) == 1 && mxGetM(arg) == 1); }
+    bool                        is_object_id(id_type *pid=0, id_type *pcid=0);
+    bool                        is_mesh_fem();
     int                         to_integer(int min_val=INT_MIN, int max_val=INT_MAX);
-    double                      to_scalar();
+    double                      to_scalar(double min_val=-1e300, double max_val=1e300);
     std::string                 to_string();
-    id_type                     to_object_id();
-    dal::bit_vector *           to_intset();
+    id_type                     to_object_id(id_type *pid=0, id_type *pcid=0);
     bgeot::base_poly *          to_poly();
-    mesh_fem_int *              to_mesh_fem();
+    getfem::mesh_fem *          to_mesh_fem();
     const getfem::getfem_mesh * to_const_mesh();
     matlabint_mesh *            to_matlabint_mesh();
+    matlabint_mesh_fem *        to_matlabint_mesh_fem();
     getfem::getfem_mesh *       to_mesh();
     getfem::pintegration_method to_fem_interpolation();
     getfem::pfem                to_fem();
     getfem::pmat_elem_type      to_mat_elem_type();
-    bgeot::pconvex_structure    to_cvs();
     bgeot::pgeometric_trans     to_pgt();
+    bgeot::pconvex_structure    to_convex_structure();
 
     /* do not perform any check on the number of dimensions of the matlab array */
     mlab_vect                   to_scalar_vector();
@@ -148,25 +196,67 @@ namespace matlabint
     mlab_vect                   to_int_vector(int expected_n, int expected_m, int expected_k=1);
 
     dal::bit_vector             to_bit_vector(const dal::bit_vector *subsetof = NULL, int shiftvals=-1);
+    getfem::base_node           to_base_node() { return to_base_node(-1); }
+    getfem::base_node           to_base_node(int expected_dim);
+    void                        to_sparse(gf_sparse_matlab_const_ref& M);
   };
 
+  /* output argument of the gf_* mex-files */
   class mexarg_out {
   public:
     mxArray *& arg;
     int argnum;
     mexarg_out(mxArray * &_arg, int _num) : arg(_arg), argnum(_num) {}
 
-    void from_object_id(id_type id);
+    void from_object_id(id_type id, id_type class_id);
+    void from_object_id(std::vector<id_type> ids, id_type class_id);
     void from_integer(int i);
     void from_scalar(double v);
+    void from_string(const char *s);
+    template<class STR_CONT> void from_string_container(const STR_CONT& s);
     void from_bit_vector(const dal::bit_vector& bv, int shift=+1);
-    void from_sparse(const gf_sparse_matrix& M);
+    void from_sparse(const gf_sparse_by_row& M, double threshold=1e-12);
+    void from_sparse(const gf_sparse_by_col& M, double threshold=1e-12);
     void from_tensor(const getfem::base_tensor& t);
     mlab_vect create_vector(unsigned dim);
     mlab_vect create_vector(unsigned n, unsigned m); 
     mlab_vect create_vector(unsigned n, unsigned m, unsigned p); 
+    template<class VECT> void from_vector(VECT& v) {
+      create_vector(v.size());
+      std::copy(v.begin(), v.end(), mxGetPr(arg));
+    }
+    template<class VEC_CONT> void from_vector_container(const VEC_CONT& vv);
   };
 
+  template<class STR_CONT> void 
+  mexarg_out::from_string_container(const STR_CONT& s)
+  {
+    arg = mxCreateCellMatrix(s.size(), 1);
+    typename STR_CONT::const_iterator it;
+    size_type cnt = 0;
+    for (it = s.begin(); it != s.end(); ++it, ++cnt) {
+      mxArray *str = mxCreateString((*it).c_str());
+      mxSetCell(arg, cnt, str);
+    }
+  }
+
+  /* output a matrix from a container of vectors (of same dimensions) */
+  template<class VEC_CONT> void 
+  mexarg_out::from_vector_container(const VEC_CONT& vv)
+  {
+    size_type n = vv.size();
+    size_type m = (n == 0) ? 0 : vv[0].size();
+    mlab_vect w  = create_vector(m,n);
+    for (size_type j=0; j < n; ++j)
+      for (size_type i=0; i < m; ++i)
+	w(i,j) = vv[j][i];
+  }
+
+  mxArray *mx_create_object_id(int nid, id_type *ids, id_type cid);
+  inline mxArray *mx_create_object_id(id_type id, id_type cid) { 
+    return mx_create_object_id(1,&id,cid); 
+  }
+
 
   /* handles the list of input arguments */
   class mexargs_in {
@@ -230,21 +320,25 @@ namespace matlabint
     bool fixed_size() const { return !use_cell; }
     mxArray **get_array() const { return out+idx; }
     void pop_all() { idx = nb_arg; }
+    void return_packed_obj_ids(const std::vector<id_type>& ids, id_type class_id);
   };
 
-  bool check_cmd(std::string& cmdname, const char *s, 
+
+  bool cmd_strmatch(const std::string& a, const char *s);
+  bool check_cmd(const std::string& cmdname, const char *s, 
 		 const mexargs_in& in, const mexargs_out& out,
 		 int min_argin=0, int max_argin=-1,
 		 int min_argout=0, int max_argout=-1);
   
   inline void bad_cmd(std::string& cmd) { 
-    DAL_THROW(matlabint_bad_arg, "Bad command name: " << cmd); }
+    THROW_BADARG("Bad command name: " << cmd); }
 
   typedef void (*mex_func_type)(mexargs_in&, mexargs_out&);
 
   void catch_errors(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[],
 		    mex_func_type pf, const char *local_workspace=NULL);
   void call_matlab_function(const std::string& fname, mexargs_in& in, mexargs_out& out);
+  void check_cv_fem(const getfem::mesh_fem& mf, size_type cv);
 
 }  /* end of namespace matlabint.                                          */
 
