@@ -36,50 +36,39 @@
 #include <matlabint.h>
 #include <getfem_export.h>
 namespace matlabint {
+  template<class SPMAT> void  smatrix_from_matlab(const mxArray *m, SPMAT& M);
+  template<> void smatrix_from_matlab<gf_sparse_by_row>(const mxArray *m, gf_sparse_by_row&);
+  template<> void smatrix_from_matlab<gf_sparse_by_col>(const mxArray *m, gf_sparse_by_col&);
 
-  
+  void  smatrix_from_matlab(const mxArray *m, gf_sparse_by_col &M);
+  mxArray * convert_to_mxSparse(const gf_sparse_by_row & smat, double threshold=1e-12);
+  mxArray * convert_to_mxSparse(const gf_sparse_by_col & smat, double threshold=1e-12);
 
-  mxArray * convert_to_mxSparse(const gf_sparse_matrix & smat);
 
-  void build_edge_list(const getfem::getfem_mesh &m, getfem::edge_list &el, mexargs_in &in);
+  void build_edge_list(const getfem::getfem_mesh &m, bgeot::edge_list &el, mexargs_in &in);
 
-  /*
-    interpolate the solution (in vector U) on the element cv, evaluated at the points 'pt'
-  */
-  template<class VECT> 
-  void interpolate_on_convex_ref(getfem::mesh_fem *mf, getfem::size_type cv, 
-				 const std::vector<getfem::base_node> &pt, 
-				 const VECT& U, size_type U_dim, 
-				 std::vector<getfem::scalar_type> &pt_val)
-  {
-    getfem::pfem cv_fem(mf->fem_of_element(cv)); assert(cv_fem != 0);
-
-    /* largely inspired by getfem_export.h */
-
-    /* interpolation of the solution.                                  */
-    /* faux dans le cas des lments non tau-equivalents ou vectoriel. */
-    assert(cv_fem->target_dim() == 1);
-    assert(cv_fem->is_equivalent());
-  
-    getfem::base_matrix G;
-
-    if (!(mf->fem_of_element(cv)->is_equivalent())) 
-      getfem::transfert_to_G(G, mf->linked_mesh().points_of_convex(cv));
-
-    for (size_type dim = 0; dim < U_dim; dim++) {
-      getfem::base_vector coeff(mf->nb_dof_of_element(cv));
-      for (getfem::size_type j = 0; j < coeff.size(); j++) {
-	coeff[j] = U[mf->ind_dof_of_element(cv)[j]*U_dim + dim];
-      }
-  
-      getfem::base_node val(1);
-      pt_val.resize(pt.size());
-      for (getfem::size_type j = 0; j < pt.size(); j++) {
-	cv_fem->interpolation(pt[j], G, mf->linked_mesh().trans_of_convex(cv), coeff, val);
-	pt_val[j*U_dim + dim] = val[0];
-      }
+  struct convex_face  {
+    size_type cv;
+    size_type f;
+    inline bool operator < (const convex_face &e) const
+    {
+      if (cv < e.cv) return true; if (cv > e.cv) return false; 
+      if (f < e.f) return true; else if (f > e.f) return false;
+      return false;
     }
-  }
+    convex_face(size_type _cv, size_type _f) : cv(_cv), f(_f) {}
+    convex_face() {}
+  };
+  void build_convex_face_lst(const getfem::getfem_mesh& m, std::vector<convex_face>& l, const mlab_vect *v);
+  void interpolate_on_convex_ref(const getfem::mesh_fem *mf, getfem::size_type cv, 
+				 const bgeot::vsvector<getfem::base_node> &pt, 
+				 const mlab_vect& U,
+				 getfem::base_matrix &pt_val);
+  void
+  eval_on_triangulated_surface(const getfem::getfem_mesh* mesh, int Nrefine,
+			       const std::vector<convex_face>& cvf,
+			       mexargs_out& out,
+			       const getfem::mesh_fem *pmf, const mlab_vect& U);
 
-};
+}
 #endif
